/**
 * Convert text to/from various formats such as CamelCase and underscore_case
 *
 * @category  WeBuilder Plugin
 * @package   CamelCase
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2016
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.0
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     35   function ToUpperCamelCase()
 *     44   function ToLowerCamelCase()
 *     54   function ToUnCamelCase()
 *     63   function ToUnderscoreCase()
 *     74   function UnCamelCase(str)
 *     85   function CamelCase(str)
 *    106   function UnderscoreCase(str)
 *    130   function Split(str, splitChar)
 *    151   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 9
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Converts selection to UowerCamelCase.
 *
 * @return void
 */
function ToUpperCamelCase() {
    Editor.SelText = CamelCase(Editor.SelText);
}

/**
 * Converts selection to lowerCamelCase.
 *
 * @return void
 */
function ToLowerCamelCase() {
    var str = CamelCase(Editor.SelText);
    Editor.SelText = Lowercase(Copy(str, 1, 1)) + Copy(str, 2, Length(str)-1)
}

/**
 * Converts selection from "CamelCase" format to "Camel Case" format.
 *
 * @return void
 */
function ToUnCamelCase() {
    Editor.SelText = UnCamelCase(Editor.SelText);
}

/**
 * Converts selection to underscore_format.
 *
 * @return void
 */
function ToUnderscoreCase() {
    Editor.SelText = UnderscoreCase(UnCamelCase(Editor.SelText));
}

/**
 * Converts string in "CamelCase" format to "Camel Case" format.
 *
 * @param  string   str
 *
 * @return string
 */
function UnCamelCase(str) {
    return RegexReplace(str, "([a-z]{1})([A-Z]{1})", "$1 $2", false);
}

/**
 * Converts string to UpperCamelCase format.
 *
 * @param  string   str
 *
 * @return string
 */
function CamelCase(str) {
    var res = "";

    // Convert "-", "_", tabs and spaces into single space
    str = RegexReplace(str, "[\\-_\\s]+", " ", true);

    // Split into single words
    str = Split(str, " ");
    for (var i=0;i<Length(str);i++) {
        res+= Uppercase(Copy(str[i], 1, 1)) + Copy(str[i], 2, Length(str[i])-1);
    }
    return res;
}

/**
 * Converts string to underscore_case format.
 *
 * @param  string   str
 *
 * @return string
 */
function UnderscoreCase(str) {
    var res = "";

    // Convert "-", "_", tabs and spaces into single space
    str = RegexReplace(str, "[\\-_\\s]+", " ", true);

    // Split into single words
    str = Split(str, " ");
    for (var i=0;i<Length(str);i++) {
        res+= "_" + Lowercase(str[i]);
    }
    // Remove first underscore
    DeleteStr(res, 1, 1);

    return res;
}

/**
 * Helper function: Split string.
 *
 * @param  string  str: The string to split
 * @param  string  splitChar: The character to split at
 * @return array
 */
function Split(str, splitChar) {
	var res[0],
	    count = 0,
		val = str + splitChar,
	   	p = Pos(splitChar,val);
	while (p > 0) {
		arg = Trim(Copy(val, 1, p - 1));
		DeleteStr(val, 1, p)
	 	SetLength(res,count + 1);	// Increase array size
		res[count] = arg;
		count++;
		p = Pos(splitChar, val);
	}
	return res;
}

/**
 * Show info when plugin is installed.
 *
 * @return void
 */
function OnInstalled() {
    Alert("CamelCase 1.0 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.RegisterDocumentAction("Change Character Case", "Convert to lowerCamelCase", "", "ToLowerCamelCase");
Script.RegisterDocumentAction("Change Character Case", "Convert to UpperCamelCase", "", "ToUpperCamelCase");
Script.RegisterDocumentAction("Change Character Case", "Convert to Un Camel Case", "", "ToUnCamelCase");
Script.RegisterDocumentAction("Change Character Case", "Convert to underscore_case", "", "ToUnderscoreCase");
